#define __PROG_TYPES_COMPAT__
#define F_CPU 16000000

#include <avr/pgmspace.h>
#include <avr/io.h>
#include <util/delay.h>
#include "uTFT_ST7735.h"
#include <avr/interrupt.h>
#include <stdio.h>
#include <stdlib.h>
#include "rtc.h"


#define C_Ds1307ReadMode_U8   0xD1u  // DS1307 ID
#define C_Ds1307WriteMode_U8  0xD0u  // DS1307 ID

uint16_t background_color = Color565(0, 51, 102);
uint8_t portii_pe_zi;
uint8_t food_per_week;
rtc_t last_feeding, current_time;
uint8_t did_reset_this_week;

void set_duty_cycle(float f) {
	OCR1A = f * ICR1;
}

void rotate_servo(float angle) {
	const static float duty_cycle_0 = 0.024;
	const static float duty_cycle_180 = 0.117;
	
	float duty_cycle =
		angle / 180 * (duty_cycle_180 - duty_cycle_0) + duty_cycle_0;
	
	set_duty_cycle(duty_cycle);
}


void pwm_init() {
	DDRD |= (1 << PD4);
	DDRD |= (1 << PD5);

	// Fast PWM, TOP ICR1
	TCCR1A |= (2 << WGM10);
	TCCR1B |= (3 << WGM12);

	// Clear OC1A, OC1B on Compare Match and set at bottom
	TCCR1A |= (1 << COM1A1);
	TCCR1A |= (1 << COM1B1);

	// Prescaler of 8
	TCCR1B |= (1 << CS11);

	// Frequency of 50 HZ
	ICR1 = 40000;
}


void servo_init() {
	pwm_init();
}

/* intreruperea pentru pin PA7 - microswitch */
ISR(PCINT0_vect) {
    // PA7 apasat
    if((PINA & (1 << PA7)) == 0)
    {
		if (current_time.min - last_feeding.min > 1 || current_time.min < last_feeding.min) {
			servo_init();
			rotate_servo(100);
			_delay_ms(2000);
			rotate_servo(220);
			_delay_ms(2000);
			DDRD &= ~(1 << PD5);
			
			portii_pe_zi ++;
			if (portii_pe_zi > 5)
				portii_pe_zi = 1;
			last_feeding = current_time;
			char str_portii[100];
			sprintf(str_portii, "Portii pe zi: %d/5", portii_pe_zi);
			setCursor(10,48);
			print(str_portii);
			
			if (current_time.weekDay == 1 && !did_reset_this_week) {
				did_reset_this_week = 1;
				food_per_week = 0;
			}
			food_per_week += 20;
			
			char str_food[100];
			sprintf(str_food, "Mancare: %dg/saptamana", food_per_week);
			setCursor(10,60);
			print(str_food);
			
			if (current_time.weekDay == 7) {
				did_reset_this_week = 0;
			}
		}
		
    }
}

void IO_init() {
    // microswitch-ul este pus ca intrare, cu rezistenta de pull-up
    DDRA &= ~(1 << PA7);
    PORTA |= (1 << PA7);
    
    PCICR  = (1 << PCIE0);
    PCMSK0 = (1 << PCINT6) | (1 << PCINT7);
}

int get_bit_from_byte(int byte, int bit) {
	return byte & (1 << bit);
}

void display_character(uint16_t x, uint16_t y, char character, uint16_t c_color, uint16_t bg_color) {
  int i, j, byte, starting_index = character * 5;
  
  for (i = 0; i < 8; i++) {
    for (j = 0; j < 5; j++) {
      byte = font[starting_index + j];
	  
      if (get_bit_from_byte(byte, i)) {
			drawPixel(i, j, c_color);
      } 
	  
	  else {
		drawPixel(i, j, bg_color);
      }
    }
  }
}

void display_string(uint16_t x, uint16_t y, char *s, uint16_t s_color, uint16_t bg_color){
	int offset = 0;
	
	while (*s != '\0'){
		display_character(x - offset, y, *s, s_color, bg_color);
		offset += 6;
		s++;
	}
	
}

void set_RTC_time(rtc_t *rtc) {

	rtc->hour = 0x21;
    rtc->min =  0x45;
    rtc->sec =  0x00;
    rtc->date = 0x25;
    rtc->month = 0x05;
    rtc->year = 0x17;
    rtc->weekDay = 4;

    RTC_SetDateTime(rtc);
}

int main(void){
	// init the 1.8 lcd display
	init();
	
	IO_init();

	fillScreen(Color565(255,255,255));
	setTextColor(Color565(0,0,0),Color565(255,255,255));
	char to_print[100];
	
	rtc_t rtc;
	
	char str_portii[100];
	sprintf(str_portii, "Portii pe zi: %d/5", portii_pe_zi);
	setCursor(10,48);
	print(str_portii);
	char str_food[100];
	sprintf(str_food, "Mancare: %dg/saptamana", food_per_week);
	setCursor(10,60);
	print(str_food);


    /*Connect SCL->PC0, SDA->PC1*/    
    RTC_Init();

    //set_RTC_time(&rtc);

	
	sei();
    while(1)

    {

        RTC_GetDateTime(&rtc);
		setCursor(10,20);
		sprintf(to_print, "Date: %2x/%2x", (uint16_t)rtc.date,(uint16_t)rtc.month);
		print(to_print);
		
		sprintf(to_print, "Time: %2x:%2x:%2x",(uint16_t)rtc.hour,(uint16_t)rtc.min,(uint16_t)rtc.sec);
		setCursor(10,32);
		print(to_print);
		
		current_time = rtc;

		_delay_ms(100);
    }
	

	return 0;
}